#include "BasicAIModule.h"
#include "../Addons/Util.h"
using namespace BWAPI;


void BasicAIModule::onStart()
{
	//initializes
	BWAPI::Broodwar->setLocalSpeed(0);
	SparCraft::init();

	Options::BotModes::SetBotMode(Options::BotModes::AIIDE_TOURNAMENT);
	Options::Modules::checkOptions();
	
    if (Options::Modules::USING_GAMECOMMANDER)
	{
		BWTA::readMap();
		BWTA::analyze();
	}
	
	if (Options::Modules::USING_MICRO_SEARCH)
	{
		SparCraft::init();
		
		//micro.onStart();
	}

	if (Options::Modules::USING_BUILD_LEARNER)
	{
		BuildOrderSearch::getStarcraftDataInstance().init(BWAPI::Broodwar->self()->getRace());
		SparCraft::Hash::initHash();
	}
	setNumWorkerScouts(0);
	setCurrentScout(NULL);

	this->showManagerAssignments=false;
	if (Broodwar->isReplay()) return;
	// Enable some cheat flags
	Broodwar->enableFlag(Flag::UserInput);

	BWTA::readMap();
	BWTA::analyze();
	this->analyzed=true;
	this->unitGroupManager   = UnitGroupManager::create();
	this->enhancedUI         = new EnhancedUI();
	this->display            = new Display();
	this->debugmanager       = new DebugManager(false);
	InformationManager::Instance().onStart();
	AttackPlanner::Instance().init();

	/*BWAPI::Race race = Broodwar->self()->getRace();
	BWAPI::Race enemyRace = Broodwar->enemy()->getRace();
	BWAPI::UnitType workerType=race.getWorker();
	double minDist;
	BWTA::BaseLocation* natural=NULL;
	BWTA::BaseLocation* home=BWTA::getStartLocation(Broodwar->self());
	for(std::set<BWTA::BaseLocation*>::const_iterator b=BWTA::getBaseLocations().begin();b!=BWTA::getBaseLocations().end();b++)
	{
		if (*b==home) continue;
		double dist=home->getGroundDistance(*b);
		if (dist>0)
		{
			if (natural==NULL || dist<minDist)
			{
				minDist=dist;
				natural=*b;
			}
		}
	}*/
}

BasicAIModule::~BasicAIModule()
{
	UnitGroupManager::destroy();
	delete this->enhancedUI;
	delete this->display;
	scoutManager.~ScoutManager();
}
void BasicAIModule::onEnd(bool isWinner)
{
	log("onEnd(%d)\n",isWinner);
}
void BasicAIModule::onFrame()
{
	if (Broodwar->isReplay()) return;
	if (!this->analyzed) return;
	
	setValidUnits();
	if( scoutUnits.empty() ){
		setScoutUnits();
	}

	timerManager.startTimer(TimerManager::All);
	//utility updates
	
	this->debugmanager->write("enhancedUI");
	this->enhancedUI->update();
	this->debugmanager->write("display");
	this->display->update();

	timerManager.startTimer(TimerManager::Worker);
	this->debugmanager->write("workerMan");
	WorkerManager::Instance().update();
	timerManager.stopTimer(TimerManager::Worker);

	timerManager.startTimer(TimerManager::Production);
	this->debugmanager->write("productionMan");
	ProductionManager::Instance().update();
	timerManager.stopTimer(TimerManager::Production);

	timerManager.startTimer(TimerManager::Building);
	this->debugmanager->write("buildMan");
	BuildingManager::Instance().update();
	timerManager.stopTimer(TimerManager::Building);

	timerManager.startTimer(TimerManager::Scout);
	if (Options::Modules::USING_SCOUTMANAGER && !scoutUnits.empty())
	{
		this->debugmanager->write("scoutMan");
		scoutManager.update(scoutUnits);
		scoutUnits.clear();
	}
	timerManager.stopTimer(TimerManager::Scout);

	timerManager.startTimer(TimerManager::InformationManager);
	this->debugmanager->write("infoMan");
	InformationManager::Instance().update();
	timerManager.stopTimer(TimerManager::InformationManager);

	timerManager.startTimer(TimerManager::MapGrid);
	MapGrid::Instance().update();
	this->debugmanager->write("mapGrid");
	timerManager.stopTimer(TimerManager::MapGrid);

	timerManager.startTimer(TimerManager::MapTools);
	this->debugmanager->write("mapTool");
	MapTools::Instance().update();
	timerManager.stopTimer(TimerManager::MapTools);

	this->arbitrator.update();

	timerManager.startTimer(TimerManager::Search);
	this->debugmanager->write("SBSearch");
	StarcraftBuildOrderSearchManager::Instance().update(35 - timerManager.getTotalElapsed());
	timerManager.stopTimer(TimerManager::Search);


	//this->debugmanager->write("attackPlanner");
	AttackPlanner::Instance().update();

	timerManager.stopTimer(TimerManager::All);

	std::set<Unit*> units=Broodwar->self()->getUnits();
	if (this->showManagerAssignments)
	{
		for(std::set<Unit*>::iterator i=units.begin();i!=units.end();i++)
		{
			if (this->arbitrator.hasBid(*i))
			{
				int x=(*i)->getPosition().x();
				int y=(*i)->getPosition().y();
				std::list< std::pair< Arbitrator::Controller<BWAPI::Unit*,double>*, double> > bids=this->arbitrator.getAllBidders(*i);
				int y_off=0;
				bool first = false;
				const char activeColor = '\x07', inactiveColor = '\x16';
				char color = activeColor;
				for(std::list< std::pair< Arbitrator::Controller<BWAPI::Unit*,double>*, double> >::iterator j=bids.begin();j!=bids.end();j++)
				{
					Broodwar->drawTextMap(x,y+y_off,"%c%s: %d",color,j->first->getShortName().c_str(),(int)j->second);
					y_off+=15;
					color = inactiveColor;
				}
			}
		}
	}

	/*UnitGroup myPylonsAndGateways = SelectAll()(Pylon,Gateway)(HitPoints,"<=",200);
	for each(Unit* u in myPylonsAndGateways)
	{
		Broodwar->drawCircleMap(u->getPosition().x(),u->getPosition().y(),20,Colors::Red);
	}*/
}

void BasicAIModule::onUnitCreate(BWAPI::Unit* unit){
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitCreate");
	InformationManager::Instance().onUnitCreate(unit);
	AttackPlanner::Instance().onUnitCreate(unit);
}
void BasicAIModule::onUnitShow(BWAPI::Unit* unit){
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitShow");
	InformationManager::Instance().onUnitShow(unit);
	WorkerManager::Instance().onUnitShow(unit);
	AttackPlanner::Instance().onUnitShow(unit);
}
void BasicAIModule::onUnitHide(BWAPI::Unit* unit){
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitHide");
	InformationManager::Instance().onUnitHide(unit);
}
void BasicAIModule::onUnitDestroy(BWAPI::Unit* unit)
{
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitDestroy");
	this->arbitrator.onRemoveObject(unit);
	InformationManager::Instance().onUnitDestroy(unit);
	WorkerManager::Instance().onUnitDestroy(unit);
	AttackPlanner::Instance().onUnitDestroy(unit);

	if(unit == currentScout) setCurrentScout(NULL);
}
void BasicAIModule::onUnitDiscover(BWAPI::Unit* unit)
{
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitDiscover");
	InformationManager::Instance().onUnitShow(unit);
	this->unitGroupManager->onUnitDiscover(unit);
}
void BasicAIModule::onUnitEvade(BWAPI::Unit* unit)
{
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitEvade");
	InformationManager::Instance().onUnitHide(unit);
	this->unitGroupManager->onUnitEvade(unit);
}
void BasicAIModule::onUnitMorph(BWAPI::Unit* unit)
{
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitMorph");
	this->unitGroupManager->onUnitMorph(unit);
	InformationManager::Instance().onUnitMorph(unit);
}
void BasicAIModule::onUnitRenegade(BWAPI::Unit* unit)
{
	if (Broodwar->isReplay()) return;
	this->debugmanager->write("onUnitRenegade");
	this->unitGroupManager->onUnitRenegade(unit);
	InformationManager::Instance().onUnitRenegade(unit);
}
void BasicAIModule::onSendText(std::string text)
{
	if (Broodwar->isReplay())
	{
		Broodwar->sendText("%s",text.c_str());
		return;
	}
	UnitType type=UnitTypes::getUnitType(text);
	if (text=="debug")
	{
		if (this->showManagerAssignments==false)
		{
			this->showManagerAssignments=true;
			//this->buildOrderManager->setDebugMode(true);
			//this->scoutManager->setDebugMode(true);
		}
		else
		{
			this->showManagerAssignments=false;
			//this->buildOrderManager->setDebugMode(false);
			//this->scoutManager->setDebugMode(false);
		}
		Broodwar->printf("%s",text.c_str());
		return;
	}
	if (text=="expand")
	{
		Broodwar->printf("%s",text.c_str());
		return;
	}
	if (type!=UnitTypes::Unknown)
	{
		//this->buildOrderManager->buildAdditional(1,type,300);
	}
	else
	{
		TechType type=TechTypes::getTechType(text);
		if (type!=TechTypes::Unknown)
		{
			//this->techManager->research(type);
		}
		else
		{
			UpgradeType type=UpgradeTypes::getUpgradeType(text);
			if (type!=UpgradeTypes::Unknown)
			{
				//this->upgradeManager->upgrade(type);
			}
			else
				Broodwar->printf("You typed '%s'!",text.c_str());
		}
	}
	Broodwar->sendText("%s",text.c_str());
}


//-------------------------------------------------//
bool isValidUnit(BWAPI::Unit * unit)
{
	if (!unit)
	{
		return false;
	}

	if (unit->isCompleted() 
		&& unit->getHitPoints() > 0 
		&& unit->exists() 
		&& unit->getType() != BWAPI::UnitTypes::Unknown 
		&& unit->getPosition().x() != BWAPI::Positions::Unknown.x() 
		&& unit->getPosition().y() != BWAPI::Positions::Unknown.y()) 
	{
		return true;
	}
	else
	{
		return false;
	}
}
const bool BasicAIModule::isAssigned(BWAPI::Unit * unit) const
{
	return assignedUnits.find(unit) != assignedUnits.end();
}
BWAPI::Unit * BasicAIModule::getFirstSupplyProvider()
{
	BWAPI::Unit * supplyProvider = NULL;

	if (BWAPI::Broodwar->self()->getRace() == BWAPI::Races::Zerg)
	{
		BOOST_FOREACH(BWAPI::Unit * unit, BWAPI::Broodwar->self()->getUnits())
		{
			if (unit->getType() == BWAPI::UnitTypes::Zerg_Spawning_Pool)
			{
				supplyProvider = unit;
			}
		}
	}
	else
	{
		
		BOOST_FOREACH(BWAPI::Unit * unit, BWAPI::Broodwar->self()->getUnits())
		{
			if (unit->getType() == BWAPI::Broodwar->self()->getRace().getSupplyProvider())
			{
				supplyProvider = unit;
			}
		}
	}

	return supplyProvider;
}
BWAPI::Unit * BasicAIModule::getClosestUnitToTarget(BWAPI::UnitType type, BWAPI::Position target)
{
	BWAPI::Unit * closestUnit = NULL;
	double closestDist = 100000;

	BOOST_FOREACH (BWAPI::Unit * unit, validUnits)
	{
		if (unit->getType() == type)
		{
			double dist = unit->getDistance(target);
			if (!closestUnit || dist < closestDist)
			{
				closestUnit = unit;
				closestDist = dist;
			}
		}
	}

	return closestUnit;
}
BWAPI::Unit * BasicAIModule::getClosestWorkerToTarget(BWAPI::Position target)
{
	BWAPI::Unit * closestUnit = NULL;
	double closestDist = 100000;

	BOOST_FOREACH (BWAPI::Unit * unit, validUnits)
	{
		if (!isAssigned(unit) && unit->getType().isWorker())
		{
			double dist = unit->getDistance(target);
			if (!closestUnit || dist < closestDist)
			{
				closestUnit = unit;
				closestDist = dist;
			}
		}
	}

	return closestUnit;
}
void BasicAIModule::setScoutUnits(){
	// if we have just built our first suply provider, set the worker to a scout
	// every 6840 frame(about 4min 30sec:24fps), set the worker to a scout
	if (numWorkerScouts == 0)
	{
		// get the first supply provider we come across in our units, this should be the first one we make
		BWAPI::Unit * supplyProvider = getFirstSupplyProvider();

		// if it exists
		if (supplyProvider)
		{
			// grab the closest worker to the supply provider to send to scout
			BWAPI::Unit * workerScout = getClosestWorkerToTarget(supplyProvider->getPosition());

			// if we find a worker (which we should) add it to the scout vector
			if (workerScout)
			{
				numWorkerScouts++;
				scoutUnits.insert(workerScout);
				assignedUnits.insert(workerScout);
				setCurrentScout(workerScout);
				WorkerManager::Instance().setScoutWorker(workerScout);
			}
		}
	}
}
void BasicAIModule::setCurrentScout(BWAPI::Unit* unit){
	currentScout = unit;
}
void BasicAIModule::setNumWorkerScouts(int num){
	numWorkerScouts = num;
}
void BasicAIModule::setValidUnits()
{
	validUnits.clear();
	assignedUnits.clear();

	// make sure the unit is completed and alive and usable
	BOOST_FOREACH(BWAPI::Unit * unit, BWAPI::Broodwar->self()->getUnits())
	{
		if (isValidUnit(unit))
		{	
			validUnits.insert(unit);
		}
	}
}